var ModelData = {

    FindFirst: function(oControl, sKey, sValue, sOper, callBack) {
        var aResults = ModelData.Find(oControl, sKey, sValue, sOper);
        if (aResults.length > 0) {
            if (typeof callBack === 'function') {
                callBack(aResults[0]);
            }
            return aResults[0];
        } else {
            if (typeof callBack === 'function') {
                callBack({});
            }
            return {};
        }
    },

    LookupValue: function(oControl, sKey, sValue, sField, sOper, callBack) {
        var aResults = ModelData.Find(oControl, sKey, sValue, sOper, callBack);
        if (aResults.length > 0) {
            return aResults[0][sField];
        } else {
            return sValue;
        }
    },

    FindDB: function(obj, table, queryString, queryData, callBack) {
        if (typeof AppDB === "undefined") {
            if (typeof callBack === "function") {
                callBack();
            }
            return;
        }
        var model = this.getModel(obj);
        var newArr = new Array();
        if (typeof queryData === 'string') {
            var qData = [];
            qData[0] = queryData;
        } else {
            var qData = queryData;
        }
        AppDB.transaction(function(tx) {
            tx.executeSql('SELECT * FROM ' + table + ' WHERE ' + queryString, qData,
                function(tx, results) {
                    for (var i = 0; i < results.rows.length; i++) {
                        var obj = results.rows.item(i);
                        for (var p in obj) {
                            if (obj[p] === "false") {
                                obj[p] = false;
                            }
                            if (obj[p] === "true") {
                                obj[p] = true;
                            }
                        }
                        newArr.push(obj);
                    }
                    model.setData(newArr);
                    if (typeof callBack === "function") {
                        callBack();
                    }
                }, null);
        }, function(tx) {
            if (tx.code !== "0") {
                callBack();
            }
        });
    },
    _Compare: function(lookIn, lookFor, oper) {
        var bCompare = false;
        switch (oper) {
            case 'Contains':
                bCompare = lookIn.indexOf(lookFor) != -1;
                break;

            case 'NE':
                bCompare = lookIn != lookFor;
                break;

            case 'GT':
                bCompare = lookIn > lookFor;
                break;

            case 'GE':
                bCompare = lookIn >= lookFor;
                break;

            case 'LT':
                bCompare = lookIn < lookFor;
                break;

            case 'LE':
                bCompare = lookIn <= lookFor;
                break;

            case 'BT':
                if ($.isArray(lookFor) && lookFor.length == 2) {
                    bCompare = lookIn >= lookFor[0] && lookIn <= lookFor[1];
                }
                break;

            case 'StartsWith':
                bCompare = lookFor.toString().length <= lookIn.toString().length &&
                    lookFor.toString() == lookIn.toString().substr(0, lookFor.toString().length);
                break;

            case 'EndsWith':
                bCompare = lookFor.toString().length <= lookIn.toString().length &&
                    lookFor.toString() == lookIn.toString().substr(lookIn.toString().length - lookFor.toString().length, lookFor.toString().length);
                break;

            default: // EQ
                bCompare = lookIn == lookFor;
                break;
        }
        return bCompare;
    },
    _CompareObjWithArray: function(obj, key, val, oper) {
        var bCompare = true;
        key.forEach(function(keyElem, IDX, ARR) {
            bCompare = bCompare && ModelData._Compare(obj[keyElem], val[IDX], oper[IDX]);
        });
        return bCompare;
    },
    _CompareObjWithObj: function(obj, key, objComp, oper) {
        var bCompare = true;
        key.forEach(function(keyElem, IDX, ARR) {
            bCompare = bCompare && ModelData._Compare(obj[keyElem], objComp[keyElem], oper[IDX]);
        });
        return bCompare;
    },
    // FIND
    Find: function(obj, key, val, oper, callBack) {
        var model = this.getModel(obj);
        var arr = model.oData;
        var ret = [];
        if (!$.isArray(arr) || arr.length == 0) return ret;

        // create array for all parameters, to have only one codeline for handling
        if (!$.isArray(key)) key = key === undefined ? [] : [key];
        if (!$.isArray(val)) val = val === undefined ? [] : [val];
        if (typeof(oper) == "undefined") oper = 'EQ'; // Default = EQ
        if (!$.isArray(oper)) {
            var operSingle = oper;
            oper = [];
            key.forEach(function(ELEM, IDX, ARR) {
                oper.push(operSingle);
            });
        }

        if (key.length != val.length || key.length != oper.length) return ret; // parameter arrays not equal size

        if (key.length === 0) {
            ret = arr;
        } else {
            ret = arr.filter(function(arrObj) {
                return ModelData._CompareObjWithArray(arrObj, key, val, oper);
            });
        }

        if (typeof callBack === 'function') {
            callBack(ret);
        }

        // Memory Clearing
        arr = null;
        return ret;
    },
    // DELETE
    Delete: function(obj, key, val, oper) {
        var model = this.getModel(obj);
        var arr = model.oData;
        if (!$.isArray(arr) || arr.length == 0) return;

        // create array for all parameters, to have only one codeline for handling
        if (!$.isArray(key)) key = key === undefined ? [] : [key];
        if (!$.isArray(val)) val = val === undefined ? [] : [val];
        if (typeof(oper) == "undefined") oper = 'EQ'; // Default = EQ
        if (!$.isArray(oper)) {
            var operSingle = oper;
            oper = [];
            key.forEach(function(ELEM, IDX, ARR) {
                oper.push(operSingle);
            });
        }
        if (key.length != val.length || key.length != oper.length) return; // parameter arrays not equal size

        if (key.length === 0) {
            arr = [];
        } else {
            for (var ai, i = arr.length; i--;) {
                if (ModelData._CompareObjWithArray(arr[i], key, val, oper)) {
                    arr.splice(i, 1);
                }

            }
        }
        model.setData(arr);
    },
    // UPDATE SINGLE FIELD
    UpdateField: function(obj, key, val, updKey, updVal, oper) {
        var model = this.getModel(obj);
        var arr = model.oData;
        if (typeof(updKey) == "undefined") return;

        // create array for all parameters, to have only one codeline for handling
        if (!$.isArray(key)) key = key === undefined ? [] : [key];
        if (!$.isArray(val)) val = val === undefined ? [] : [val];
        if (!$.isArray(updKey)) updKey = updKey === undefined ? [] : [updKey];
        if (!$.isArray(updVal)) updVal = updVal === undefined ? [] : [updVal];
        if (typeof(oper) == "undefined") oper = 'EQ'; // Default = EQ
        if (!$.isArray(oper)) {
            var operSingle = oper;
            oper = [];
            key.forEach(function(ELEM, IDX, ARR) {
                oper.push(operSingle);
            });
        }
        if (key.length != val.length || key.length != oper.length) return; // parameter arrays not equal size
        if (updKey.length != updVal.length) return;

        for (var ai, i = arr.length; i--;) {
            if (ModelData._CompareObjWithArray(arr[i], key, val, oper)) {
                updKey.forEach(function(ELEM, IDX, ARR) {
                    arr[i][ELEM] = updVal[IDX];
                })
            }
        }
        model.setData(arr);
    },
    // ADD
    Add: function(obj, rec) {
        var model = this.getModel(obj);
        var arrObj = [];
        if (typeof(model.oData) != 'undefined' && model.oData.length) {
            arrObj = model.oData;
        }
        arrObj.push(rec);
        model.setData(arrObj);

        // Memory Clear
        model = null;
        arrObj = null;
    },
    //  ADD ARRAY
    AddArray: function(obj, arr) {
        var model = this.getModel(obj);
        var arrObj = [];
        if (typeof(model.oData) != 'undefined' && model.oData.length) {
            arrObj = model.oData;
        }
        $.each(arr, function(i, data) {
            arrObj.push(data);
        });
        model.setData(arrObj);
    },
    //  UPDATE
    Update: function(obj, key, val, rec, oper) {
        var model = this.getModel(obj);
        var mData = model.oData;
        var upd = false;
        // First record
        if (typeof mData == 'undefined') {
            var mData = new Array();
        }

        // create array for all parameters, to have only one codeline for handling
        if (!$.isArray(key)) key = key === undefined ? [] : [key];
        if (!$.isArray(val)) val = val === undefined ? [] : [val];
        if (typeof(oper) == "undefined") oper = 'EQ'; // Default = EQ
        if (!$.isArray(oper)) {
            var operSingle = oper;
            oper = [];
            key.forEach(function(ELEM, IDX, ARR) {
                oper.push(operSingle);
            });
        }
        if (key.length != val.length || key.length != oper.length) return; // parameter arrays not equal size


        // Update
        if (key.length > 0) {
            for (var ai, i = mData.length; i--;) {

                if (ModelData._CompareObjWithArray(mData[i], key, val, oper)) {
                    mData[i] = rec;
                    upd = true;
                }
            }
        }
        if (upd == true) {
            model.setData(mData);
        } else {
            ModelData.Add(obj, rec);
        }
    },
    //  UPDATE ARRAY
    UpdateArray: function(obj, key, arr, oper) {
        var model = this.getModel(obj);
        var mData = model.oData;
        var upd = false;
        // First record
        if (typeof mData.length == 'undefined') {
            var mData = new Array();
        }

        // create array for all parameters, to have only one codeline for handling
        if (!$.isArray(key)) key = key === undefined ? [] : [key];
        if (typeof(oper) == "undefined") oper = 'EQ'; // Default = EQ
        if (!$.isArray(oper)) {
            var operSingle = oper;
            oper = [];
            key.forEach(function(ELEM, IDX, ARR) {
                oper.push(operSingle);
            });
        }
        if (key.length != oper.length) return; // parameter arrays not equal size

        $.each(arr, function(i, newData) {
            upd = false;
            if (key.length > 0) {
                for (i = 0; i < mData.length; i++) {
                    if (ModelData._CompareObjWithObj(mData[i], key, newData, oper)) {
                        mData[i] = newData;
                        upd = true;
                    }
                }
            }
            if (upd === false) {
                mData.push(newData);
            }
        });
        model.setData(mData);
    },
    getModel: function(obj) {
        var model = obj.getModel();
        if (typeof model === "undefined") {
            for (var p in obj.oModels) {
                var model = obj.getModel(p);
            }
        }
        return model
    },
    //  UUID
    genID: function() {
        var d = new Date().getTime();
        var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
            var r = (d + Math.random() * 16) % 16 | 0;
            d = Math.floor(d / 16);
            return (c == 'x' ? r : (r & 0x7 | 0x8)).toString(16);
        });
        return uuid;
    }
}